verifyPasswordResetCode method

  1. @override
Future<bool> verifyPasswordResetCode({
  1. required String code,
})
override

Implementation

@override
Future<bool> verifyPasswordResetCode({required String code}) async {
  if (!isReady) {
    throw LittlefishFirebaseAuthException('AUTH_INIT_000',
        '$providerName: Firebase is not initialized, $providerName cannot verify password reset code');
  }

  logger.debug(
    this,
    "$providerName: Verifying password reset code",
  );
  try {
    await kInstance?.verifyPasswordResetCode(code);

    throw UnimplementedError(
        'verifyPasswordResetCode has not been implemented for $providerName, please complete the implementation if you want to use this service');
  } on FirebaseAuthException catch (e) {
    throw _handleFirebaseAuthException(e);
  } catch (e) {
    if (e is UnimplementedError) {
      rethrow;
    }
    if (e is LittlefishException) {
      rethrow;
    }
    throw LittlefishFirebaseAuthException('AUTH_014',
        'An unexpected error occurred during password reset code verification.',
        cause: e);
  }
}